/*
 * ServThread
 */
import java.io.*;
import java.net.*;

/**
 * ServThread is the thread which takes care of
 * the client request and the responce to that request.
 */
public class ServThread extends Thread {
    /** The TCP/IP socket */
    public Socket sok=null;
    /** The service which handles incomming communication with the client */
    private HttpRequest httpRequest;
    /** The service which handles outgoing communication with the client */
    private HttpResponse httpResponse;
    /** Wait-state support for thread pools */
    boolean wait=false;
    /* Log file */
    //private File syslog=new File(".\\access.log");

    /**
     * ServThread constructor used to store this thread
     * in the thread pool to be used when needed. It doesn't
     * do any communication untill told to do so.
     */
    public ServThread() { wait=true; }

    /**
     * ServThread constructor used to communicate
     * with a client without any delay.
     */
    public ServThread(Socket s) {
        sok=s;
    }

    public void run() {
        /* If communication with the client succeeded serve the request */
        try {
            /* Default timeout to 30 seconds for request */
            sok.setSoTimeout(30000);
            /* Create an object which represents the request from the client
             * and handles incomming communication with the client */
            httpRequest=new HttpRequest(sok);
            /* Create an object which represents our response
             * and handles outgoing communication with the client */
            httpResponse=new HttpResponse(sok,httpRequest);
            do {
                /* Flush any unsent data */
                httpResponse.flush();
                /* Reset the request in case this was a keep-alive connection */
                httpRequest.reset();
                /* Receive the request from the client */
                try { httpRequest.start(); }
                catch(IllegalAccessException e) { throw new IllegalAccessException(); }
                catch(Exception e) {
                }
                /* Check the validity of the request */
                if(httpRequest.ok==false) {
                    httpResponse.sendString(new ErrorCode(httpRequest.errorCode,httpRequest).toString());
                    httpResponse.flush();
                    /* Serve more requests if keep-alive is enabled */
                    continue;
                }
                /* Get the requested URL */
                GetURL url=new GetURL(httpRequest);
                /* Check that the URL is accessible */
                if(url.ok==false) {
                    httpResponse.sendString(new ErrorCode(url.errorCode,httpRequest).toString());
                    httpResponse.flush();
                    /* Serve more requests if keep-alive is enabled */
                    continue;
                }
                /* Respond based on the request type */
                if(httpRequest.httpMethod.equalsIgnoreCase("GET")) {
                    /* Support for HTTP/0.9 based on RFC 1945, section 4.1 */
                    if(!url.isDirectory&url.isCGI==false) {
                        if(httpRequest.httpVersion>=1.0)
                            httpResponse.sendFileHeaders(url,httpRequest.errorCode,httpRequest.httpVersion);
                        httpResponse.sendFile(url);
                    } else if(!url.isDirectory&url.isCGI==true) {
                        httpResponse.sendCGI(url,httpRequest,sok);
                        /* No persistanse for CGI scripts */
                        throw new IOException();
                    } else {
                        if(httpRequest.httpVersion>=1.0)
                            httpResponse.sendListingHeaders(url,httpRequest.errorCode,httpRequest.httpVersion);
                        httpResponse.sendListing(url);
                        httpResponse.flush();
                        /* We always close the connection after a file listing */
                        throw new IOException();
                    }
                    httpResponse.flush();
                    continue;
                } else if(httpRequest.httpMethod.equalsIgnoreCase("HEAD")) {
                    httpResponse.sendFileHeaders(url,httpRequest.errorCode,httpRequest.httpVersion);
                    httpResponse.flush();
                    continue;
                } else {
                    httpResponse.sendString(new ErrorCode(501,httpRequest).toString());
                    httpResponse.flush();
                    continue;
                }
            } while(httpRequest.connection.equalsIgnoreCase("keep-alive"));
        } catch(Exception e) { }
        try { sok.close(); } catch(Exception e) { }
    }

}
